from visual import *
from visual.graph import *
from random import random

# A model of an ideal gas with hard-sphere collisions
# Program uses numpy arrays for high speed computations

# Bruce Sherwood

win=500   # the size of screen

Natoms = 100  # change this to have more or fewer atoms

# Typical values
L = 1. # container is a cube L on a side
gray = (1.7,0.7,0.7) # color of edges of container
Matom = 4E-3/6E23 # helium mass
Ratom = 0.014 # wildly exaggerated size of helium atom
k = 1.4E-23 # Boltzmann constant
T = 300. # around room temperature
dt = 1E-2
#mass=1

#****************** The properties of screen
scene = display(title="Gas", width=win, height=win, x=0, y=0,
                center=(L/2.,L/2.,L/2.))  # the title and shape of screen
deltav = 100. # binning for v histogram
vdist = gdisplay(x=0, y=win, ymax =Natoms*deltav/1000. ,
             width=win, height=0.6*win, xtitle='v', ytitle='dN')
#***************************************



theory = gcurve(color=color.cyan)  # show a line with a color
dv = 10.
for v in arange(0.,3001.+dv,dv): # theoretical prediction
    theory.plot(pos=(v,(deltav/dv)*Natoms*4.*pi*((Matom/(2.*pi*k*T))**1.5)*exp((-0.5*Matom*v**2)/(k*T))*v**2*dv))
 
#*****************************************************

    
#observation = ghistogram(bins=arange(0.,3000.,deltav),accumulate=1, average=1, color=color.red)
#**************** Box properties ******************
xaxis = curve(pos=[(0,0,0), (L,0,0)], color=gray)
yaxis = curve(pos=[(0,0,0), (0,L,0)], color=gray)
zaxis = curve(pos=[(0,0,0), (0,0,L)], color=gray)
xaxis2 = curve(pos=[(L,L,L), (0,L,L), (0,0,L), (L,0,L)], color=gray)
yaxis2 = curve(pos=[(L,L,L), (L,0,L), (L,0,0), (L,L,0)], color=gray)
zaxis2 = curve(pos=[(L,L,L), (L,L,0), (0,L,0), (0,L,L)], color=gray)

#**************************
Atoms = []
ball = []
colors = [color.red, color.green, color.blue,
          color.yellow, color.cyan, color.magenta]
poslist = []
plist = []
mlist = []
rlist = []
pos1 = []
poslist1 = []
pos = []
position = [[[0 for k in range(100)] for j in range(100)]
            for i in range(100)]
rr = 0

mass=1
pavg =1 #sqrt(2.*mass*1.5*k*T) # average kinetic energy p**2/(2mass) = (3/2)kT
Lmin = 2.0*Ratom
lll= Ratom
Lmax = L-Lmin
llmin=0
llmax=2.0*Ratom*int(L/(2*Ratom))
r = Ratom
#print Lmin,Lmax,L
#print nx,ny,nz,position[nx][ny][nz]
#pos = array(poslist)
#print pos[1,0]
#print position
rr=0
for i in range(Natoms):
    mass=1
    pavg =10
    theta = pi*random()
    phi = 2*pi*random()
    px = pavg*sin(theta)*cos(phi)
    py = pavg*sin(theta)*sin(phi)
    pz = pavg*cos(theta)
    x = llmax*random()
    y = llmax*random()
    z = llmax*random()
    nx=int(x/(2*Ratom))+1
    ny=int(y/(2*Ratom))+1
    nz=int(z/(2*Ratom))+1
    print nx,ny,nz

    rr = 0
    while rr==0:
        if position[nx][ny][nz]==0:
            xx=(2*nx-1)*Ratom
            yy=(2*ny-1)*Ratom
            zz=(2*nz-1)*Ratom

            poslist.append((xx,yy,zz))
            position[nx][ny][nz]=1
            Atoms = Atoms+[sphere(pos=(xx,yy,zz), radius=r, color=colors[i % 6])]
            rr=1
            plist.append((px,py,pz))
            mlist.append(mass)
            rlist.append(r)
        if position[nx][ny][nz]==1:
           x = llmax*random()
           y = llmax*random()
           z = llmax*random()
           nx=int(x/(2*Ratom))+1
           ny=int(y/(2*Ratom))+1
           nz=int(z/(2*Ratom))+1
        
            

    
pos = array(poslist)
p = array(plist)
m = array(mlist)
t=0
pos = pos+(p/mass)*(dt/2.)

#print pos[2,0]
    
out=0
rd=0.0
while 1:  #t<10 and out == 0: # and sqrt(pos.pos)<=L/2.0:
    rate(1/dt)
    #observation.plot(data=mag(p/m))
    for i in range(Natoms):
      pos[i,0] = pos[i,0]+(p[i,0]/mass)*(dt/2.0)
      pos[i,1] = pos[i,1]+(p[i,1]/mass)*(dt/2.0)
      pos[i,2] = pos[i,2]+(p[i,2]/mass)*(dt/2.0)
      Atoms[i].pos = pos[i]
#***** wall collision **************      
      #if abs(Atoms[i].pos.x)>=L:
      if pos[i,0]>=L-Ratom or pos[i,0]<=Ratom:
          p[i,0]=-p[i,0] 
      if pos[i,1]>=L-Ratom or pos[i,1]<=Ratom:
          p[i,1]=-p[i,1]
      if pos[i,2]>=L-Ratom or pos[i,2]<=Ratom:
          p[i,2]=-p[i,2]
#***********************************
#********* pair collision************
    for i in range(Natoms):
        for j in range(i+1,Natoms): 
            distance=sqrt((pos[i,0]-pos[j,0])**2+(pos[i,1]-pos[j,1])**2
                          +(pos[i,2]-pos[j,2])**2)
            if distance<=Ratom*2:
                delta_vix=((p[j,0]-p[i,0])*(pos[i,0]-pos[j,0])+(p[j,1]-p[i,1])*(pos[i,1]-pos[j,1])+
                 (p[j,2]-p[i,2])*(pos[i,2]-pos[j,2]))*(pos[i,0]-pos[j,0])/distance**2
                delta_viy=((p[j,0]-p[i,0])*(pos[i,0]-pos[j,0])+(p[j,1]-p[i,1])*(pos[i,1]-pos[j,1])+
                 (p[j,2]-p[i,2])*(pos[i,2]-pos[j,2]))*(pos[i,1]-pos[j,1])/distance**2
                delta_viz=((p[j,0]-p[i,0])*(pos[i,0]-pos[j,0])+(p[j,1]-p[i,1])*(pos[i,1]-pos[j,1])+
                 (p[j,2]-p[i,2])*(pos[i,2]-pos[j,2]))*(pos[i,2]-pos[j,2])/distance**2
                delta_vjx=-delta_vix
                delta_vjy=-delta_viy
                delta_vjz=-delta_viz
                p[i,0]=p[i,0]+delta_vix
                p[i,1]=p[i,1]+delta_viy
                p[i,2]=p[i,2]+delta_viz
                p[j,0]=p[j,0]+delta_vjx
                p[j,1]=p[j,1]+delta_vjy
                p[j,2]=p[j,2]+delta_vjz
                 
          
    #print pos[i]      
    t=t+dt
print t,rd,out    

